@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: module.c,v 1.2.32.2 2010/08/12 21:58:31 phargrov Exp $
+ * $Id: module.c,v 1.2.32.6 2012/12/23 04:36:34 phargrov Exp $
  */
 
 #include "blcr_config.h"
@@ -88,13 +88,13 @@ static int __init blcr_imports_init(void)
 	/* Check current kernel against System.map used at configure time */
 	{
 #if defined(CR_EXPORTED_KCODE_register_chrdev)
-	    unsigned long offset1 = CR_EXPORTED_KCODE_register_chrdev - (unsigned long)&register_chrdev;
+	    volatile unsigned long offset1 = CR_EXPORTED_KCODE_register_chrdev - (unsigned long)&register_chrdev;
 #elif defined(CR_EXPORTED_KCODE___register_chrdev)
-	    unsigned long offset1 = CR_EXPORTED_KCODE___register_chrdev - (unsigned long)&__register_chrdev;
+	    volatile unsigned long offset1 = CR_EXPORTED_KCODE___register_chrdev - (unsigned long)&__register_chrdev;
 #else
 	    #error "No register_chrdev symbol for validation of System.map"
 #endif
-	    unsigned long offset2 = CR_EXPORTED_KCODE_register_blkdev - (unsigned long)&register_blkdev;
+	    volatile unsigned long offset2 = CR_EXPORTED_KCODE_register_blkdev - (unsigned long)&register_blkdev;
 	    if (
 #if defined(CONFIG_RELOCATABLE) && defined(CONFIG_PHYSICAL_ALIGN)
 	    /* Check that relocation offset is valid and constant */
