@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_chkpt_req.c,v 1.264.6.4 2011/08/23 21:41:49 phargrov Exp $
+ * $Id: cr_chkpt_req.c,v 1.264.6.5 2012/12/18 18:32:05 phargrov Exp $
  */
 
 #include "cr_module.h"
@@ -59,6 +59,10 @@ static int release_request(cr_chkpt_req_t *req)
 // release_proc_req()
 //
 // Try to free a proc_req and associated resources
+//
+// Must be called w/ (parent) req->lock held for writing or when
+// exclusive access to the proc_req is otherwise guaranteed.
+// TODO: atomic ref_count instead of locking parent?
 static void release_proc_req(cr_chkpt_proc_req_t *proc_req)
 {
 	proc_req->ref_count -= 1;
@@ -1329,7 +1333,9 @@ out_no_delete:
 		CR_KTRACE_LOW_LVL("blocking for proces to complete (count=%d)", atomic_read(&proc_req->predump_barrier.count));
 		CR_ASSERT_STEP_EQ(cr_task, 0); // Was reset by delete_dead_task
 		cr_barrier_wait_interruptible(&proc_req->predump_barrier);
+		write_lock(&req->lock);
 		release_proc_req(proc_req);
+		write_unlock(&req->lock);
 		(void)release_request(req);
 	}
 
