@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_context.h,v 1.72.8.1 2011/09/22 23:22:14 eroman Exp $
+ * $Id: cr_context.h,v 1.72.8.2 2012/12/19 05:21:28 phargrov Exp $
  */
 
 /*
@@ -188,7 +188,7 @@ struct cr_open_dir {
 };
 
 struct cr_chrdev {
-    cr_obj_t cr_type; /* cr_open_chr */
+    cr_obj_t cr_type; /* cr_chrdev_obj */
     unsigned int cr_major;
     unsigned int cr_minor;
     mode_t i_mode;
@@ -196,7 +196,7 @@ struct cr_chrdev {
 };
 
 struct cr_dup {
-    cr_obj_t cr_type; /* cr_open_dup */
+    cr_obj_t cr_type; /* cr_dup_obj */
     /* nothing more */
 };
 
