@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_dump_self.c,v 1.228.8.4 2011/08/24 02:23:58 phargrov Exp $
+ * $Id: cr_dump_self.c,v 1.228.8.7 2012/12/19 05:21:28 phargrov Exp $
  */
 
 #include "cr_module.h"
@@ -262,9 +262,9 @@ cr_get_fd_info(struct files_struct *files, int fd, struct cr_file_info *file_inf
 
     rcu_read_lock();
     fdt = cr_fdtable(files);
-    if (FD_ISSET(fd, fdt->open_fds)) {
+    if (cr_read_open_fd(fd, fdt)) {
         file_info->fd      = fd;
-        file_info->cloexec = FD_ISSET(fd, fdt->close_on_exec);
+        file_info->cloexec = cr_read_close_on_exec(fd, fdt);
         file_info->orig_filp = fcheck(fd);
     } else {
         CR_WARN("cr_get_fd_info: Called on closed file!");
@@ -461,7 +461,7 @@ cr_save_open_chr(cr_chkpt_proc_req_t *proc_req, struct file *filp)
 
     inode = filp->f_dentry->d_inode;
 
-    cf_chrdev.cr_type  = cr_open_chr;
+    cf_chrdev.cr_type  = cr_chrdev_obj;
     if (cr_task_tty(current) && (cr_task_tty(current) == (struct tty_struct *)filp->private_data)) {
 	/* Map CTTY -> /dev/tty */
 	cf_chrdev.cr_major = TTYAUX_MAJOR;
@@ -510,7 +510,7 @@ cr_save_open_dup(cr_chkpt_proc_req_t *proc_req, struct file *filp)
       goto out;
 
     /* placeholder... just in case we need to do something later */
-    cf_dup.cr_type  = cr_open_dup;
+    cf_dup.cr_type  = cr_dup_obj;
 
     retval = cr_kwrite(eb, cf_filp, &cf_dup, sizeof(cf_dup));
     if (retval != sizeof(cf_dup)) {
