@@ -21,15 +21,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_fops.c,v 1.69.8.3 2011/09/29 21:22:09 phargrov Exp $
+ * $Id: cr_fops.c,v 1.69.8.4 2012/12/18 18:32:05 phargrov Exp $
  */
 
 #include "cr_module.h"
 
-#if HAVE_FILE_OPERATIONS_UNLOCKED_IOCTL
-  #include <linux/smp_lock.h>
-#endif
-
 /**
  * cr_hand_complete - wrapper/helper for CR_OP_HAND_DONE
  * @file: (struct file *) of the control node
@@ -143,10 +139,6 @@ static int ctrl_ioctl(struct inode *inode, struct file *file,
 
 	CR_KTRACE_FUNC_ENTRY("op=%08x arg=0x%lx", op, arg);
 
-#if HAVE_FILE_OPERATIONS_UNLOCKED_IOCTL
-	lock_kernel();
-#endif
-
 	switch(op) {
 	//
 	// Calls from libcr:
@@ -255,10 +247,6 @@ static int ctrl_ioctl(struct inode *inode, struct file *file,
 		CR_KTRACE_BADPARM("unknown op %x", _IOC_NR(op));
 	}
 
-
-#if HAVE_FILE_OPERATIONS_UNLOCKED_IOCTL
-	unlock_kernel();
-#endif
 	return result;
 }
 
