@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_pipes.c,v 1.225.8.7 2011/09/28 23:10:22 eroman Exp $
+ * $Id: cr_pipes.c,v 1.225.8.9 2013/01/04 04:21:24 phargrov Exp $
  */
 
 #include "cr_module.h"
@@ -81,7 +81,7 @@ cr_posix_flags_to_f_mode(int posix_flags)
  * ensures the second will not block for lack of readers or writers.
  */
 static int
-cr_open_named_fifo(cr_errbuf_t *eb, int fd, struct dentry *dentry, struct vfsmount *mnt, int flags)
+cr_open_named_fifo(cr_errbuf_t *eb, int fd, struct path *path, int flags)
 {
     int retval = 0;
     struct file *filp, *rw_filp;
@@ -92,11 +92,8 @@ cr_open_named_fifo(cr_errbuf_t *eb, int fd, struct dentry *dentry, struct vfsmou
      * know this "extra" RDWR open will work regardless of true access rights.
      * THIS IS ALSO WHY WE MUST CLOSE rw_filp WHEN DONE WITH IT.
      */
-  #if HAVE_TASK_CRED
-    rw_filp = dentry_open(dget(dentry), mntget(mnt), O_RDWR, cr_current_cred());
-  #else
-    rw_filp = dentry_open(dget(dentry), mntget(mnt), O_RDWR);
-  #endif
+    path_get(path);
+    rw_filp = cr_dentry_open(path, O_RDWR);
     retval = PTR_ERR(rw_filp);
     if (IS_ERR(rw_filp)) goto out;
 	
@@ -173,7 +170,12 @@ cr_couple_named_fifo(cr_rstrt_req_t *req, struct cr_file_info *file_info,
 	retval = -EIO;
 	goto out_dput;
     }
-    retval = cr_open_named_fifo(eb, file_info->fd, new_dentry, mnt, cf_fifo->fifo_flags);
+    {
+	struct path path;
+	path.dentry = new_dentry;
+	path.mnt = mnt;
+	retval = cr_open_named_fifo(eb, file_info->fd, &path, cf_fifo->fifo_flags);
+    }
 
 out_dput:
     dput(new_dentry);
@@ -184,7 +186,7 @@ out:
 /* Set up a new file pointer to an existing pipe */
 static int
 cr_couple_unnamed_pipe(cr_rstrt_req_t *req, struct cr_file_info *file_info, 
-	struct cr_fifo *cf_fifo, const struct file *first_filp)
+	struct cr_fifo *cf_fifo, struct file *first_filp)
 {
     int retval;
     struct file *filp;
@@ -224,29 +226,26 @@ cr_creat_named_fifo(cr_errbuf_t *eb, struct cr_file_info *file_info,
 		    const struct cr_fifo *cf_fifo, const char *name)
 {
     int retval;
-    struct nameidata nd;
-    struct dentry * dentry;
+    struct path path;
 
     /* make a new FIFO, or find the existing one */
-    dentry = cr_mknod(eb, &nd, name, cf_fifo->fifo_perms, file_info->unlinked ? file_info->fd+1 : 0);
-    if (IS_ERR(dentry)) {
-	retval = PTR_ERR(dentry);
+    retval = cr_mknod(eb, &path, name, cf_fifo->fifo_perms, file_info->unlinked ? file_info->fd+1 : 0);
+    if (retval) {
 	CR_ERR_EB(eb, "Failed to create named fifo %s.  err=%d.", name, retval);
 	goto out;
     }
 
     /* open it */
-    retval = cr_open_named_fifo(eb, file_info->fd, dentry, nd.nd_mnt, cf_fifo->fifo_flags);
+    retval = cr_open_named_fifo(eb, file_info->fd, &path, cf_fifo->fifo_flags);
     if (retval < 0) {
 	CR_ERR_EB(eb, "Couldn't open %s.  err=%d.", name, retval);
-	goto out_dput;
+	goto out_put;
     }
 
     retval = 0;
 
-out_dput:
-    dput(dentry);
-    cr_path_release(&nd);
+out_put:
+    path_put(&path);
 out:
     return retval;
 }
