@@ -21,7 +21,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: cr_rstrt_req.c,v 1.393.4.11 2011/09/17 00:16:29 phargrov Exp $
+ * $Id: cr_rstrt_req.c,v 1.393.4.16 2012/12/25 03:27:28 phargrov Exp $
  */
 
 #include "cr_module.h"
@@ -115,13 +115,13 @@ alloc_rstrt_req(void)
 	    fdt = cr_fdtable(files);
 	    /* Dup stdin, unless marked CLOEXEC */
 	    tmp = fcheck_files(files, 0);
-	    if (tmp && !FD_ISSET(0, fdt->close_on_exec)) {
+	    if (tmp && !cr_read_close_on_exec(0, fdt)) {
 		req->dpipe_in = tmp;
 		get_file(tmp);
 	    }
 	    /* Dup stdout, unless marked CLOEXEC */
 	    tmp = fcheck_files(files, 1);
-	    if (tmp && !FD_ISSET(1, fdt->close_on_exec)) {
+	    if (tmp && !cr_read_close_on_exec(1, fdt)) {
 		req->dpipe_out = tmp;
 		get_file(tmp);
 	    }
@@ -1595,7 +1595,7 @@ cr_restore_open_chr(cr_rstrt_proc_req_t *proc_req, struct cr_file_info *file_inf
 
     /* XXX:  Validate type */
     retval = -EINVAL;
-    if (cf_chrdev.cr_type != cr_open_chr) {
+    if (cf_chrdev.cr_type != cr_chrdev_obj) {
         CR_ERR_PROC_REQ(proc_req, "cr_restore_open_chr: Garbage in context file.");
         goto out;
     }
@@ -1730,7 +1730,7 @@ cr_restore_open_dup(cr_rstrt_proc_req_t *proc_req, struct cr_file_info *file_inf
 
     /* Validate type (the only field we've got) */
     retval = -EINVAL;
-    if (cf_dup.cr_type != cr_open_dup) {
+    if (cf_dup.cr_type != cr_dup_obj) {
         CR_ERR_PROC_REQ(proc_req, "cr_restore_open_dup: Garbage in context file.");
         goto out;
     }
@@ -1855,7 +1855,7 @@ cr_restore_all_files(cr_rstrt_proc_req_t *proc_req)
     spin_lock(&files->file_lock);
     fdt = cr_fdtable(files);
     for (i = 0; i < fdt->max_fds; ++i) {
-	if (FD_ISSET(i, fdt->close_on_exec)) {
+	if (cr_read_close_on_exec(i, fdt)) {
 	    spin_unlock(&files->file_lock);
 	    sys_close(i);
 	    spin_lock(&files->file_lock);
@@ -1945,9 +1945,9 @@ cr_restore_all_files(cr_rstrt_proc_req_t *proc_req)
         spin_lock(&files->file_lock);
 	fdt = cr_fdtable(files);
         if (file_info.cloexec) {
-	    FD_SET(file_info.fd, fdt->close_on_exec);
+	    cr_set_close_on_exec(file_info.fd, fdt);
         } else {
-	    FD_CLR(file_info.fd, fdt->close_on_exec);
+	    cr_clear_close_on_exec(file_info.fd, fdt);
 	}
 	spin_unlock(&files->file_lock);
 
@@ -1974,29 +1974,27 @@ out:
  * cr_chdir
  *
  * Our own version of the chdir() system call.  Differs in that it accepts
- * pathnames in kernel space, and calls path_init, path_walk, and path_release
- * directly.
+ * pathnames in kernel space, and performs lookup directly.
  * 
  */
 static int
-cr_chdir(const char *path)
+cr_chdir(const char *name)
 {
-    struct nameidata nd;
     int retval;
+    struct path path;
 
-    /* lookup the path */
-    retval = path_lookup(path,LOOKUP_FOLLOW|LOOKUP_DIRECTORY,&nd);
+    retval = cr_kern_path(name, LOOKUP_FOLLOW|LOOKUP_DIRECTORY, &path);
     if (retval)
         goto out;
 
-    retval = cr_permission(nd.nd_dentry->d_inode,MAY_EXEC,&nd);
+    retval = cr_permission(path.dentry->d_inode, MAY_EXEC|MAY_CHDIR);
     if (retval)
-        goto dput_and_out;
+        goto out_put;
 
-    cr_set_pwd_nd(current->fs, &nd);
+    cr_set_pwd_path(current->fs, &path);
 
-dput_and_out:
-    cr_path_release(&nd);
+out_put:
+    path_put(&path);
 out:
     return retval;
 }
@@ -2080,7 +2078,7 @@ cr_hide_filp(struct file *filp, struct files_struct *files)
 	if (filp == fdt->fd[fd]) {
 	    ++found;
             rcu_assign_pointer(fdt->fd[fd], NULL);
-	    FD_CLR(fd, fdt->close_on_exec);
+	    cr_clear_close_on_exec(fd, fdt);
 	    break;
 	}
     }
@@ -2315,6 +2313,11 @@ static inline int cr_free_audit_context(void)
     #elif HAVE_2_6_6_AUDIT_SYSCALL_EXIT
 	/* linux-2.6.6 through 2.6.11 */
         audit_syscall_exit(current, 0);
+    #elif HAVE___AUDIT_SYSCALL_EXIT
+	/* EL6 kernels */
+        if (unlikely(current->audit_context)) {
+                __audit_syscall_exit(1, 0);
+        }
     #else
         #error "Don't know how to call audit_syscall_exit()"
     #endif
@@ -2391,6 +2394,7 @@ int cr_rstrt_child(struct file *filp)
     down(&proc_req->serial_mutex);
     if (req->die) {
 	up(&proc_req->serial_mutex);
+	retval = 0; /* don't clobber error code from whoever set req->die */
 	goto out_release;
     }
 
@@ -2995,7 +2999,7 @@ int
 cr_rstrt_init(void)
 {
     int retval = 0;
-    cr_linkage_cachep = KMEM_CACHE(cr_linkage_s, 0);
+    cr_linkage_cachep = CR_KMEM_CACHE(cr_linkage_s);
     if (!cr_linkage_cachep) {
 	retval = -ENOMEM;
     }
