@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: vmadump_ppc64.c,v 1.9.14.1 2011/09/16 00:26:08 phargrov Exp $
+ * $Id: vmadump_ppc64.c,v 1.9.14.4 2012/12/22 07:42:52 phargrov Exp $
  *
  * THIS VERSION MODIFIED FOR BLCR <http://ftg.lbl.gov/checkpoint>
  *-----------------------------------------------------------------------*/
@@ -29,6 +29,10 @@
 #define __VMADUMP_INTERNAL__
 #include "vmadump.h"
 
+#if HAVE_ASM_SWITCH_TO_H
+#  include <asm/switch_to.h>
+#endif
+
 long vmadump_store_cpu(cr_chkpt_proc_req_t *ctx, struct file *file,
 		       struct pt_regs *regs) {
     long bytes = 0, r;
@@ -164,9 +168,12 @@ loff_t vmad_store_arch_map(cr_chkpt_proc_req_t *ctx, struct file *file,
 	head.end     = map->vm_end;
 	head.flags   = map->vm_flags;
 	head.namelen = VMAD_NAMELEN_ARCH;
-	head.offset  = 0;
+	head.pgoff   = 0;
 
+	up_read(&current->mm->mmap_sem);
 	r = write_kern(ctx, file, &head, sizeof(head));
+	down_read(&current->mm->mmap_sem);
+
 	if (r < 0) return r;
 	if (r != sizeof(head)) r = -EIO;
     }
